import { GraphQueryResultFromBackend } from '../data-access/store/graphQueryResultSlice';
import { GraphStatistics } from './statistics.types';
import { getAttributeType, initializeStatistics, updateStatistics } from './utils';

const getGraphStatistics = (graph: GraphQueryResultFromBackend): GraphStatistics => {
  const { nodes, edges } = graph;

  const n_nodes = nodes.length;
  const n_edges = edges.length;

  const density = n_nodes < 2 ? 0 : (n_edges * 2) / (n_nodes * (n_nodes - 1));

  // general nodes and edges statistics
  const metaData: GraphStatistics = {
    topological: { density, self_loops: 0 },
    nodes: { labels: [], count: n_nodes, types: {} },
    edges: { labels: [], count: n_edges, types: {} },
  };

  // attributes based statistics
  nodes.forEach((node) => {
    const nodeType = node.label;
    if (!metaData.nodes.labels.includes(nodeType)) {
      metaData.nodes.labels.push(nodeType);
    }

    if (!metaData.nodes.types[nodeType]) {
      metaData.nodes.types[nodeType] = { count: 0, attributes: {} };
    }
    metaData.nodes.types[nodeType].count++;

    Object.entries(node.attributes).forEach(([attributeId, attributeValue]) => {
      const attributeType = getAttributeType(attributeValue);

      if (!metaData.nodes.types[nodeType].attributes[attributeId]) {
        metaData.nodes.types[nodeType].attributes[attributeId] = { attributeType, statistics: initializeStatistics(attributeType) };
      }
      updateStatistics(metaData.nodes.types[nodeType].attributes[attributeId], attributeValue);
    });
  });

  // Process edges
  edges.forEach((edge) => {
    const edgeType = edge.label;
    if (!metaData.edges.labels.includes(edgeType)) {
      metaData.edges.labels.push(edgeType);
    }

    if (!metaData.edges.types[edgeType]) {
      metaData.edges.types[edgeType] = { count: 0, attributes: {} };
    }

    metaData.edges.types[edgeType].count++;

    if (edge.from === edge.to) metaData.topological.self_loops++;

    Object.entries(edge.attributes).forEach(([attributeId, attributeValue]) => {
      const attributeType = getAttributeType(attributeValue);

      if (!metaData.edges.types[edgeType].attributes[attributeId]) {
        metaData.edges.types[edgeType].attributes[attributeId] = { attributeType, statistics: initializeStatistics(attributeType) };
      }

      updateStatistics(metaData.edges.types[edgeType].attributes[attributeId], attributeValue);
    });
  });

  return metaData;
};

export { getGraphStatistics };
